@@ -79,15 +79,15 @@ static struct {
 } gl/*obal*/;
 
 typedef enum {
-    CONNECT_STEP_FIRST,
-    CONNECT_STEP_CHECK_RUNNING,
-    CONNECT_STEP_STACK_INIT,
-    CONNECT_STEP_CHECK_NSS,
-    CONNECT_STEP_IPSEC_START,
-    CONNECT_STEP_WAIT_READY,
-    CONNECT_STEP_CONFIG_ADD,
-    CONNECT_STEP_CONNECT,
-    CONNECT_STEP_LAST
+	CONNECT_STEP_FIRST,
+	CONNECT_STEP_CHECK_RUNNING,
+	CONNECT_STEP_STACK_INIT,
+	CONNECT_STEP_CHECK_NSS,
+	CONNECT_STEP_IPSEC_START,
+	CONNECT_STEP_WAIT_READY,
+	CONNECT_STEP_CONFIG_ADD,
+	CONNECT_STEP_CONNECT,
+	CONNECT_STEP_LAST
 } ConnectStep;
 
 typedef struct {
@@ -107,6 +107,7 @@ typedef struct {
 	gboolean interactive;
 	gboolean pending_auth;
 	gboolean managed;
+	gboolean xauth_enabled;
 
 	GPid pid;
 	guint watch_id;
@@ -247,28 +248,41 @@ typedef struct {
 } ValidProperty;
 
 static ValidProperty valid_properties[] = {
-	{ NM_LIBRESWAN_RIGHT,                      G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_LEFTID,                     G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_LEFTXAUTHUSER,              G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_DOMAIN,                     G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_DHGROUP,                    G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_PFSGROUP,                   G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_DPDTIMEOUT,                 G_TYPE_INT, 0, 86400 },
-	{ NM_LIBRESWAN_IKE,                        G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_ESP,                        G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_VENDOR,                     G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_RIGHT,                      G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_RIGHTID,                    G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_RIGHTRSASIGKEY,             G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_LEFT,                       G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_LEFTID,                     G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_LEFTXAUTHUSER,              G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_LEFTRSASIGKEY,              G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_LEFTCERT,                   G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_DOMAIN,                     G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_DHGROUP,                    G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_PFSGROUP,                   G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_DPDTIMEOUT,                 G_TYPE_INT, 0, 86400 },
+	{ NM_LIBRESWAN_KEY_IKE,                        G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_ESP,                        G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_IKELIFETIME,                G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_SALIFETIME,                 G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_VENDOR,                     G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_REMOTENETWORK,              G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_IKEV2,                      G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_NARROWING,                  G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_REKEY,                      G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_FRAGMENTATION,              G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_MOBIKE,                     G_TYPE_STRING, 0, 0 },
 	/* Ignored option for internal use */
-	{ NM_LIBRESWAN_PSK_INPUT_MODES,            G_TYPE_NONE, 0, 0 },
-	{ NM_LIBRESWAN_XAUTH_PASSWORD_INPUT_MODES, G_TYPE_NONE, 0, 0 },
-	{ NM_LIBRESWAN_PSK_VALUE "-flags",         G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_XAUTH_PASSWORD "-flags",    G_TYPE_STRING, 0, 0 },
-	{ NULL,                                   G_TYPE_NONE, 0, 0 }
+	{ NM_LIBRESWAN_KEY_PSK_INPUT_MODES,            G_TYPE_NONE, 0, 0 },
+	{ NM_LIBRESWAN_KEY_XAUTH_PASSWORD_INPUT_MODES, G_TYPE_NONE, 0, 0 },
+	{ NM_LIBRESWAN_KEY_PSK_VALUE "-flags",         G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_XAUTH_PASSWORD "-flags",    G_TYPE_STRING, 0, 0 },
+	{ NULL,                                    G_TYPE_NONE, 0, 0 }
 };
 
 static ValidProperty valid_secrets[] = {
-	{ NM_LIBRESWAN_PSK_VALUE,                  G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_XAUTH_PASSWORD,             G_TYPE_STRING, 0, 0 },
-	{ NULL,                                   G_TYPE_NONE, 0, 0 }
+	{ NM_LIBRESWAN_KEY_PSK_VALUE,                  G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_XAUTH_PASSWORD,             G_TYPE_STRING, 0, 0 },
+	{ NULL,                                    G_TYPE_NONE, 0, 0 }
 };
 
 typedef struct ValidateInfo {
@@ -483,25 +497,25 @@ connect_cleanup (NMLibreswanPlugin *self)
 		priv->retry_id = 0;
 	}
 
-	if (priv->io_id) {
-		g_source_remove (priv->io_id);
-		priv->io_id = 0;
-	}
-	g_clear_pointer (&priv->channel, g_io_channel_unref);
+	if (priv->xauth_enabled) {
+		if (priv->io_id) {
+			g_source_remove (priv->io_id);
+			priv->io_id = 0;
+		}
+		g_clear_pointer (&priv->channel, g_io_channel_unref);
 
-	if (priv->io_buf) {
-		g_string_free (priv->io_buf, TRUE);
-		priv->io_buf = NULL;
+		if (priv->io_buf) {
+			g_string_free (priv->io_buf, TRUE);
+			priv->io_buf = NULL;
+		}
+		if (priv->password) {
+			memset (priv->password, 0, strlen (priv->password));
+			g_free (priv->password);
+			priv->password = NULL;
+		}
 	}
-
 	pipe_cleanup (&priv->out);
 	pipe_cleanup (&priv->err);
-
-	if (priv->password) {
-		memset (priv->password, 0, strlen (priv->password));
-		g_free (priv->password);
-		priv->password = NULL;
-	}
 }
 
 static void
@@ -716,11 +730,11 @@ nm_libreswan_config_psk_write (NMSettingVpn *s_vpn,
 	gboolean success;
 
 	/* Check for ignored group password */
-	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_PSK_INPUT_MODES);
+	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_PSK_INPUT_MODES);
 	if (pw_type && !strcmp (pw_type, NM_LIBRESWAN_PW_TYPE_UNUSED))
 		return TRUE;
 
-	psk = nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_PSK_VALUE);
+	psk = nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_KEY_PSK_VALUE);
 	if (!psk)
 		return TRUE;
 
@@ -751,11 +765,11 @@ nm_libreswan_config_psk_write (NMSettingVpn *s_vpn,
 		return FALSE;
 	}
 
-	leftid = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_LEFTID);
+	leftid = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTID);
 	if (leftid) {
 		success = write_config_option (fd, NULL, error, "@%s: PSK \"%s\"", leftid, psk);
 	} else {
-		right = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_RIGHT);
+		right = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_RIGHT);
 		g_assert (right);
 		success = write_config_option (fd, NULL, error, "%s %%any: PSK \"%s\"", right, psk);
 	}
@@ -969,7 +983,7 @@ badpipes:
  * specific subnets to be routed over the VPN (eg, CISCO_SPLIT_INC).
  * This is what we need to automatically determine 'never-default' behavior.
  * Instead, we have to inspect the kernel's SAD (Security Assocation Database)
- * for IPSec-secured routes pointing to the VPN gateway.
+ * for IPsec-secured routes pointing to the VPN gateway.
  */
 
 typedef struct {
@@ -1445,7 +1459,7 @@ handle_auth (NMLibreswanPlugin *self, const char **out_message, const char **out
 		g_free (priv->password);
 		priv->password = NULL;
 	} else {
-		*out_hint = NM_LIBRESWAN_XAUTH_PASSWORD;
+		*out_hint = NM_LIBRESWAN_KEY_XAUTH_PASSWORD;
 		*out_message = _("A password is required.");
 	}
 
@@ -1552,8 +1566,8 @@ connect_step (NMLibreswanPlugin *self, GError **error)
 
 	switch (priv->connect_step) {
 	case CONNECT_STEP_FIRST:
-		/* fall through */
 		priv->connect_step++;
+		/* fallthrough */
 
 	case CONNECT_STEP_CHECK_RUNNING:
 		if (!do_spawn (self, &priv->pid, NULL, NULL, error, priv->ipsec_path, "auto", "--status", NULL))
@@ -1575,8 +1589,8 @@ connect_step (NMLibreswanPlugin *self, GError **error)
 				priv->watch_id = g_child_watch_add (priv->pid, child_watch_cb, self);
 			return success;
 		}
-		/* fall through */
 		priv->connect_step++;
+		/* fallthrough */
 
 	case CONNECT_STEP_CHECK_NSS:
 		/* Start the IPsec service */
@@ -1587,8 +1601,8 @@ connect_step (NMLibreswanPlugin *self, GError **error)
 				priv->watch_id = g_child_watch_add (priv->pid, child_watch_cb, self);
 			return success;
 		}
-		/* fall through */
 		priv->connect_step++;
+		/* fallthrough */
 
 	case CONNECT_STEP_IPSEC_START:
 		/* Start the IPsec service */
@@ -1651,18 +1665,23 @@ connect_step (NMLibreswanPlugin *self, GError **error)
 	}
 	case CONNECT_STEP_CONNECT:
 		g_assert (uuid);
-		if (!spawn_pty (self, &up_stdout, &up_stderr, &up_pty, &priv->pid, error,
-		                priv->ipsec_path, "auto", "--up", uuid, NULL))
+
+		if (!spawn_pty (self, &up_stdout, &up_stderr,
+		                priv->xauth_enabled ? &up_pty : NULL,
+		                &priv->pid, error,
+		                priv->ipsec_path, "auto", "--up", uuid, NULL)) {
 			return FALSE;
+		}
 		priv->watch_id = g_child_watch_add (priv->pid, child_watch_cb, self);
 
-		/* Wait for the password request */
-		priv->io_buf = g_string_sized_new (128);
-		priv->channel = g_io_channel_unix_new (up_pty);
-		g_io_channel_set_encoding (priv->channel, NULL, NULL);
-		g_io_channel_set_buffered (priv->channel, FALSE);
-		priv->io_id = g_io_add_watch (priv->channel, G_IO_IN | G_IO_ERR | G_IO_HUP, io_cb, self);
-
+		if (priv->xauth_enabled) {
+			/* Wait for the password request */
+			priv->io_buf = g_string_sized_new (128);
+			priv->channel = g_io_channel_unix_new (up_pty);
+			g_io_channel_set_encoding (priv->channel, NULL, NULL);
+			g_io_channel_set_buffered (priv->channel, FALSE);
+			priv->io_id = g_io_add_watch (priv->channel, G_IO_IN | G_IO_ERR | G_IO_HUP, io_cb, self);
+		}
 		pipe_init (&priv->out, up_stdout, "OUT");
 		pipe_init (&priv->err, up_stderr, "ERR");
 		return TRUE;
@@ -1738,7 +1757,11 @@ _connect_common (NMVpnServicePlugin   *plugin,
 		return FALSE;
 	}
 
-	priv->password = g_strdup (nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_XAUTH_PASSWORD));
+	/* XAUTH is not part of the IKEv2 standard and we always enforce it in IKEv1 */
+	priv->xauth_enabled = !nm_libreswan_utils_setting_is_ikev2 (s_vpn, NULL);
+
+	if (priv->xauth_enabled)
+		priv->password = g_strdup (nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_KEY_XAUTH_PASSWORD));
 
 	/* Write the IPsec secret (group password); *SWAN always requires this and
 	 * doesn't ask for it interactively.
@@ -1781,6 +1804,9 @@ real_need_secrets (NMVpnServicePlugin *plugin,
                    GError **error)
 {
 	NMSettingVpn *s_vpn;
+	const char *leftcert;
+	const char *leftrsasigkey;
+	const char *rightrsasigkey;
 	const char *pw_type;
 
 	g_return_val_if_fail (NM_IS_VPN_SERVICE_PLUGIN (plugin), FALSE);
@@ -1795,19 +1821,33 @@ real_need_secrets (NMVpnServicePlugin *plugin,
 		return FALSE;
 	}
 
-	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_PSK_INPUT_MODES);
+	/* When leftcert is specified, rsasigkey are assumed to be '%cert' */
+	leftcert = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTCERT);
+	if (leftcert)
+		goto xauth_check;
+
+	/* If authentication is done through rsasigkeys, only the public keys are required */
+	leftrsasigkey = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_LEFTRSASIGKEY);
+	rightrsasigkey = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_RIGHTRSASIGKEY);
+	if (leftrsasigkey && rightrsasigkey)
+		goto xauth_check;
+
+	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_PSK_INPUT_MODES);
 	if (!pw_type || strcmp (pw_type, NM_LIBRESWAN_PW_TYPE_UNUSED)) {
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_PSK_VALUE)) {
+		if (!nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_KEY_PSK_VALUE)) {
 			*setting_name = NM_SETTING_VPN_SETTING_NAME;
 			return TRUE;
 		}
 	}
 
-	pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_XAUTH_PASSWORD_INPUT_MODES);
-	if (!pw_type || strcmp (pw_type, NM_LIBRESWAN_PW_TYPE_UNUSED)) {
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_XAUTH_PASSWORD)) {
-			*setting_name = NM_SETTING_VPN_SETTING_NAME;
-			return TRUE;
+xauth_check:
+	if (!nm_libreswan_utils_setting_is_ikev2 (s_vpn, NULL)) {
+		pw_type = nm_setting_vpn_get_data_item (s_vpn, NM_LIBRESWAN_KEY_XAUTH_PASSWORD_INPUT_MODES);
+		if (!pw_type || strcmp (pw_type, NM_LIBRESWAN_PW_TYPE_UNUSED)) {
+			if (!nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_KEY_XAUTH_PASSWORD)) {
+				*setting_name = NM_SETTING_VPN_SETTING_NAME;
+				return TRUE;
+			}
 		}
 	}
 
@@ -1837,7 +1877,7 @@ real_new_secrets (NMVpnServicePlugin *plugin,
 	_LOGD ("VPN received new secrets; sending to ipsec");
 
 	g_free (priv->password);
-	priv->password = g_strdup (nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_XAUTH_PASSWORD));
+	priv->password = g_strdup (nm_setting_vpn_get_secret (s_vpn, NM_LIBRESWAN_KEY_XAUTH_PASSWORD));
 
 	g_warn_if_fail (priv->pending_auth);
 	if (!handle_auth (self, &message, &hints[0])) {
